#ifndef _VARIABLE_H_
#define _VARIABLE_H_

#include <stdio.h>
#include "error.h"
#include "radec.h"
#include "varcfg.h"

#define MAX(a, b) ((a)>(b)?(a):(b))

struct variable{
        struct variable *next;  // single linked list
        int constell, number;   // constallation and star nr in constallation
        char component;         // Component identification  
        char gcvs[11];          // Variable star designation
        char noteflag;          // Notes in published catalog
        Ra_t Ra1950;            // rectascenzion equinox 1950.0
        Dec_t Dec1950;          // declination equinox 1950.0
        Ra_t Ra;                // rectascenzion equinox 2000.0
        Dec_t Dec;              // declination equinox 2000.0
        char u_des;             // position accuracy flag
        char vartype[11];       // Type of variability

        char l_magmax;          // ">" if magMax is a faint limit
                                // "<" if magMax is a bright limit
                                // "(" if magMax is an amplitude
        float magmax;           // Magnitude at maximum brightness
        char u_magmax;          // Uncertainty flag (:) on magMax
        char f_magmax;          // ")" if magMax is an amplitude

        char l_magmin;          // "<" if magMin is a bright limit
                                // "(" if magMin is an amplitude
        float magmin;          // Minimum magnitude or amplitude
        char u_magmin;          // Uncertainty flag (:) on magMi
        char n_magmin;          // Alternative photometric system for magMin
        char f_magmin;          // ")" if magMin is an amplitude

        char magcode[3];        // The photometric system for magnitudes
        float epoch;           // Epoch for maximum light, Julian days
        char q_epoch;           // Quality flag on Epoch
        int yearnova;           // Year of outburst for nova
        char l_period;          // Code for upper or lower limits
        float period;          // Period of the variable star
        char u_period;          // Uncertainty flag (:) on Period
        char n_period[4];       // Uncertainty notes on Period
        char mmd[4];            // Rising time (M-m) or duration of eclipse (D)
        char u_mmd;             // Uncertainty flag (:) on M-m/D
        char n_mmd;             // Note for eclipsing variable
        char sptype[18];        // Spectral typ
        char ref1[6];           // Reference to a study of the star
        char ref2[6];           // Reference to a chart or photograph
        char exists[15];        // Cases of non-existence of the variable etc
};

enum{
        SEARCH_NAME = (1 << 1),
        SEARCH_MAG = (1 << 2),
        SEARCH_TYPE = (1 << 3),
        SEARCH_CONST = (1 << 4),
        SEARCH_PERIOD = (1 << 5),
        SEARCH_AMPLITUDE = (1 << 6)
};

struct variable_list{
        struct variable *head, *tail;
        size_t count;
};

typedef struct variable_list variable_t;

// load
int var_load_variables(char *filename, variable_t *varlist);

// list
variable_t *var_list_init(void);
void var_list_free(variable_t *varlist);
void var_list_insert(variable_t *varlist, struct variable *var);
void var_list_print(variable_t *varlist);

// search
int var_search_name(const char *patt, variable_t *varlist, variable_t *var);
int var_search_magmin(float patt, variable_t *varlist, variable_t *var);
int var_search_magmax(float patt, variable_t *varlist, variable_t *var);
int var_search_type(char *patt, variable_t *varlist, variable_t *var);
int var_search_const(int patt, variable_t *varlist, variable_t *var);
void var_serch_period(float patt, variable_t *varlist, FILE *fp);
char *var_search(variable_t *varlist, 
                 const char *name,
                 const char *type,
                 const char *mag,
                 const char *amp,
                 const char *constell);

// print
void var_print_object(struct variable *var, FILE *fp);
void var_print_header(FILE *fp);

// menu
int main_menu(void);
int mag_menu(variable_t *varlist);
int name_menu(variable_t *varlist);
int type_menu(variable_t *varlist);
int const_menu(variable_t *varlist);

// util
float get_float();
float get_int();
char *get_string();
FILE *get_file();


// constell
int const_get_id_by_name(const char *name);
int const_get_id_by_shrt(const char *shrt);
char *const_get_name_by_id(int id);
char *const_get_name_by_shrt(const char *shrt);
char *const_get_shrt_by_id(int id);
char *const_get_shrt_by_name(const char *name);


// other
void var_draw_percent(float percent);


#endif
